/*
 * $Id: //depot/prod/kamino/main/util/viewServerInfo/src/ViewServerCmnd.cpp#4 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */

#include "ViewServerCmnd.h"

ViewServerCmnd::ViewServerCmnd()
{
}

ViewServerCmnd::~ViewServerCmnd()
{
}

int ViewServerCmnd::bind(char* host, char* ntLogon, char* password, LDAP *&ld) {
    // initialize a session 
	WCHAR wHost[512];
	mbstowcs_s(0, wHost, 512, host, 512);
    if ((ld = ldap_init(wHost, 389)) == NULL) {
        return LDAP_CONNECT_ERROR;
	}

    // Set the LDAP version used in this session
    ULONG version = LDAP_VERSION3;
    int ret = ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, (void*)&version);
    if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_set_option failed with 0x%x\n", ret);
			fclose(out);
		}
        return ret;
    }

    // Connect to the server
    ret = ldap_connect(ld, NULL);
    if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_connect failed with 0x%x\n", ret);
			fclose(out);
		}
        return ret;
    }

    // create the credentials to bind with
    SEC_WINNT_AUTH_IDENTITY auth;
    auth.Domain = NULL;
    auth.DomainLength = 0;
    auth.User = (unsigned short*)ntLogon;
    auth.UserLength = (unsigned long)strlen(ntLogon);
    auth.Password = (unsigned short*)password;
    auth.PasswordLength = (unsigned long)strlen(password);
    auth.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;

    // bind to the LDAP server
    ret = ldap_bind_s(ld, NULL, (PWCHAR)&auth, LDAP_AUTH_NTLM);
    if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_bind_s call failed with 0x%x\n", ret);
			fclose(out);
		}
        return ret;
    }

	return LDAP_SUCCESS;
}

int ViewServerCmnd::unbind(LDAP *&ld) {
    // unbind from the LDAP server
    int ret = ldap_unbind_s(ld);
    if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_unbind_s call failed with 0x%x\n", ret);
			fclose(out);
		}
        return ret;
    }

	return LDAP_SUCCESS;
}

int ViewServerCmnd::ValidateCredentials(char* host, char* ntLogon, char* password) {
	LDAP *ld;
	int ret = bind(host, ntLogon, password, ld);
	if(ret != LDAP_SUCCESS) {
		return ret;
	}
	ret = unbind(ld);
	if(ret != LDAP_SUCCESS) {
		return ret;
	}
	return LDAP_SUCCESS;
}

int ViewServerCmnd::GetVirtualCenterGUID(char* host, char* ntLogon, char* password, vector<string> &addresses, char* dn) {
    if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering GetVirtualCenterGUID\n");
		fclose(out);
	}

    LDAP *ld;
	int ret = bind(host, ntLogon, password, ld);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap bind failed with 0x%x\n", ret);
			fclose(out);
		}
		return ret;
	}

	// Search for the VC GUID
	string filter = "(&(objectClass=pae-VirtualCenter) (|";
	for(vector<string>::size_type i = 0; i < addresses.size(); i++) {
		filter += "(pae-VCURL=*" +addresses.at(i) +"*)";
	}
	filter += "))";

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "filter: %s\n", filter.c_str());
		fclose(out);
	}

	WCHAR wFilter[512];
	mbstowcs_s(0, wFilter, 512, filter.c_str(), 512);

	LDAPMessage *msg;
	ret = ldap_search_s(ld,
		L"ou=VirtualCenter,ou=Properties,dc=vdi,dc=vmware,dc=int",
		LDAP_SCOPE_SUBTREE,
		wFilter,
		NULL,
		0,
		&msg);

	if(ret == LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_search_s succeeded\n");
			fclose(out);
		}
	} else {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_search_s call failed with 0x%x\n", ret);
			fclose(out);
		}
		return ret;
	}

    // get the VC GUID
	LDAPMessage *entry = ldap_first_entry(ld, msg);
	PWCHAR* value = ldap_get_values(ld, entry, L"CN");
	ULONG count = ldap_count_values(value);
	if(count == 0) { // there's no VC added to the View Server
		return LDAP_NO_SUCH_OBJECT;
	}

    // unbind from the LDAP server
	ret = unbind(ld);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap unbind failed with 0x%x", ret);
			fclose(out);
		}
		return ret;
	}

	wcstombs_s(0, dn, 512, value[0], 512);

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "VCDN: %s\n", dn);
		fprintf(out, "Exiting GetVirtualCenterGUID\n");
		fclose(out);
	}

	return LDAP_SUCCESS;
}

int ViewServerCmnd::GetPools(char* host,
							 char* ntLogon,
							 char* password,
							 boolean manualPoolsOnly,
							 vector<ViewPool*> &pools) {
    if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering GetPools\n");
		fclose(out);
	}

	LDAP *ld;
	int ret = bind(host, ntLogon, password, ld);
	if(ret != LDAP_SUCCESS) {
		return ret;
	}

	// Search for the pools
	LDAPMessage *msg;
	if(manualPoolsOnly) {

	    if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "Searching for pools with type 0 and 5\n");
			fclose(out);
		}

		ret = ldap_search_s(ld,
			L"ou=Server Groups,dc=vdi,dc=vmware,dc=int",
			LDAP_SCOPE_SUBTREE,
			L"(&(objectClass=pae-ServerPool) (|(pae-ServerPoolType=0)(pae-ServerPoolType=5)))",
			NULL,
			0,
			&msg);

		if(ret == LDAP_SUCCESS) {
			if(DEBUG) {
				FILE* out;
				fopen_s(&out, "debug.log", "a");
				fprintf(out, "ldap_search_s succeeded\n");
				fclose(out);
			}
		} else {
			if(DEBUG) {
				FILE* out;
				fopen_s(&out, "debug.log", "a");
				fprintf(out, "ldap_search_s call failed with 0x%x\n", ret);
				fclose(out);
			}
			return ret;
		}
	} else {

	    if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "Searching for everything where objectClass=pae-ServerPool\n");
			fclose(out);
		}

		ret = ldap_search_s(ld,
			L"ou=Server Groups,dc=vdi,dc=vmware,dc=int",
			LDAP_SCOPE_SUBTREE,
			L"(objectClass=pae-ServerPool)",
			NULL,
			0,
			&msg);

		if(ret == LDAP_SUCCESS) {
			if(DEBUG) {
				FILE* out;
				fopen_s(&out, "debug.log", "a");
				fprintf(out, "ldap_search_s succeeded\n");
				fclose(out);
			}
		} else {
			if(DEBUG) {
				FILE* out;
				fopen_s(&out, "debug.log", "a");
				fprintf(out, "ldap_search_s call failed with 0x%x\n", ret);
				fclose(out);
			}
			return ret;
		}
	}

	// get the desktop names
	LDAPMessage *entry = ldap_first_entry(ld, msg);
	while(entry != NULL) {
		PWCHAR* poolNameValue = ldap_get_values(ld, entry, L"CN");
		char poolName[512];
		wcstombs_s(0, poolName, 512, poolNameValue[0], 512);

		PWCHAR* fullVcdnValue = ldap_get_values(ld, entry, L"pae-VCDN");
		char fullVcdn[512];
		wcstombs_s(0, fullVcdn, 512, fullVcdnValue[0], 512);

		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "poolName = %s\n", poolName);
			fprintf(out, "vcdn = %s\n", fullVcdn);
			fclose(out);
		}

		string vcdn(fullVcdn);
		vcdn = vcdn.substr(3, 36); // extract only the GUID

		ViewPool* viewPool = new ViewPool();
		viewPool->setName(poolName);
		viewPool->setVcdn(vcdn);

		pools.push_back(viewPool);
		entry = ldap_next_entry(ld, entry);
	}

	// unbind from LDAP
	ret = unbind(ld);
	if(ret != LDAP_SUCCESS) {
		return ret;
	}

    if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Exiting GetPools\n");
		fclose(out);
	}
	return LDAP_SUCCESS;
}

int ViewServerCmnd::ImportPool(char* host, 
							   char* ntLogon, 
							   char* password,
							   vector<string> &addresses,
							   int accessMode, 
							   int poolType, 
							   map<string, vector<ViewDesktopSource*> > &desktopSourceMap) {
	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering ImportPool\n");
		fclose(out);
	}

	LDAP *ld;
	int ret = bind(host, ntLogon, password, ld);
	if(ret != LDAP_SUCCESS) {
		return ret;
	}

	char vcdn[512];
	ret = GetVirtualCenterGUID(host, ntLogon, password, addresses, vcdn);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "GetVirtualCenterGUID failed with 0x%x\n", ret);
			fclose(out);
		}
    	return ret;
    }

	// add each new VM to the Servers OU
	map<string, vector<ViewDesktopSource*> >::iterator poolItr;
	for(poolItr = desktopSourceMap.begin(); poolItr != desktopSourceMap.end(); poolItr++) {
		vector<ViewDesktopSource*>::iterator desktopSourceItr;
		for(desktopSourceItr = (*poolItr).second.begin(); desktopSourceItr != (*poolItr).second.end(); desktopSourceItr++) {
			ViewDesktopSource *desktopSource = (ViewDesktopSource*)(*desktopSourceItr);
			ret = addVM(desktopSource, ld);
			if(ret != LDAP_SUCCESS) {
				if(DEBUG) {
					FILE* out;
					fopen_s(&out, "debug.log", "a");
					fprintf(out, "addVM failed with 0x%x\n", ret);
					fclose(out);
				}
				return ret;
			}
		}
	}

	for(poolItr = desktopSourceMap.begin(); poolItr != desktopSourceMap.end(); poolItr++) {
		string poolName = (*poolItr).first;

		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "add/modify pool named %s\n", poolName.c_str());
			fclose(out);
		}

		// add/modify the pool
		if(poolType == 0) { // New pool
			ret = addServerGroup(accessMode, poolName, (*poolItr).second, ld, vcdn);
			if(ret != LDAP_SUCCESS) {
				if(DEBUG) {
					FILE* out;
					fopen_s(&out, "debug.log", "a");
					fprintf(out, "addServerGroup failed with 0x%x", ret);
					fclose(out);
				}
				return ret;
			}
		} else { // Existing pool
			ret = addToExistingServerGroup(poolName, (*poolItr).second, ld);
			if(ret != LDAP_SUCCESS) {
				if(DEBUG) {
					FILE* out;
					fopen_s(&out, "debug.log", "a");
					fprintf(out, "addToExistingServerGroup failed with 0x%x", ret);
					fclose(out);
				}
				return ret;
			}
		}

		// create the application entry
		if(poolType == 0) { // New pool
			ret = addApplication(poolName, ld);
			if(ret != LDAP_SUCCESS) {
				if(DEBUG) {
					FILE* out;
					fopen_s(&out, "debug.log", "a");
					fprintf(out, "addApplication failed with 0x%x", ret);
					fclose(out);
				}
				return ret;
			}
		}
	}

	ret = unbind(ld);
	if(ret != LDAP_SUCCESS) {
		return ret;
	}

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Exiting ImportPool\n");
		fclose(out);
	}

	return LDAP_SUCCESS;
}

int ViewServerCmnd::addVM(ViewDesktopSource *desktopSource, LDAP *&ld) {
	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering addVM\n");
		fclose(out);
	}

	string dnString = "CN=" +desktopSource->getName() +",OU=Servers,dc=vdi,dc=vmware,dc=int";

	LDAPMod name, objectClass, cn, distinguishedName, paeOptIgnoreProcessList, paeMoid, paeServerManaged, paeSSOEnabled, paeDisplayName,
		paeTunneledConnection, ipHostNumber, paePwdEncryption, paeClientProtVersion, paeWinDomain, paeThinProto, paeServices,
		paeVmPath, paeOptSuspendTimeout, paeOptDisconnectLimitTimeout, paeOptMaximumSessions, paeDisabled;

	// convert desktop name to wide char
	WCHAR desktopSourceName[512];
	mbstowcs_s(0, desktopSourceName, 512, desktopSource->getName().c_str(), 512);

	// convert dn to wide char
	WCHAR dn[512];
	mbstowcs_s(0, dn, 512, dnString.c_str(), 512);

	PWCHAR name_values[] = {desktopSourceName, NULL};
	name.mod_op = LDAP_MOD_ADD;
	name.mod_type = L"name";
	name.mod_values = name_values;

	PWCHAR objectClass_values[] = {L"top", L"pae-Server", L"pae-WinServer", L"pae-ThinWinServer", L"pae-VM", NULL};
	objectClass.mod_op = LDAP_MOD_ADD;
	objectClass.mod_type = L"objectClass";
	objectClass.mod_values = objectClass_values;

	PWCHAR cn_values[] = {desktopSourceName, NULL};
	cn.mod_op = LDAP_MOD_ADD;
	cn.mod_type = L"cn";
	cn.mod_values = cn_values;

	PWCHAR distinguishedName_values[] = {dn, NULL};
	distinguishedName.mod_op = LDAP_MOD_ADD;
	distinguishedName.mod_type = L"distinguishedName";
	distinguishedName.mod_values = distinguishedName_values;

	PWCHAR paeOptIgnoreProcessList_values[] = {L"0", NULL};
	paeOptIgnoreProcessList.mod_op = LDAP_MOD_ADD;
	paeOptIgnoreProcessList.mod_type = L"pae-OptIgnoreProcessList";
	paeOptIgnoreProcessList.mod_values = paeOptIgnoreProcessList_values;

	// convert MOID to wide char
	WCHAR moid[512];
	mbstowcs_s(0, moid, 512, desktopSource->getMoid().c_str(), 512);

	PWCHAR paeMoid_values[] = {moid, NULL};
	paeMoid.mod_op = LDAP_MOD_ADD;
	paeMoid.mod_type = L"pae-MOID";
	paeMoid.mod_values = paeMoid_values;

	PWCHAR paeServerManaged_values[] = {L"1", NULL};
	paeServerManaged.mod_op = LDAP_MOD_ADD;
	paeServerManaged.mod_type = L"pae-ServerManaged";
	paeServerManaged.mod_values = paeServerManaged_values;

	PWCHAR paeSSOEnabled_values[] = {L"1", NULL};
	paeSSOEnabled.mod_op = LDAP_MOD_ADD;
	paeSSOEnabled.mod_type = L"pae-SSOEnabled";
	paeSSOEnabled.mod_values = paeSSOEnabled_values;

	PWCHAR paeDisplayName_values[] = {desktopSourceName, NULL};
	paeDisplayName.mod_op = LDAP_MOD_ADD;
	paeDisplayName.mod_type = L"pae-DisplayName";
	paeDisplayName.mod_values = paeDisplayName_values;

	PWCHAR paeTunneledConnection_values[] = {L"1", NULL};
	paeTunneledConnection.mod_op = LDAP_MOD_ADD;
	paeTunneledConnection.mod_type = L"pae-TunneledConnection";
	paeTunneledConnection.mod_values = paeTunneledConnection_values;

	string ipHostNumberString = desktopSource->getName() + "." +desktopSource->getDomain();
	// convert IP host number to wide char
	WCHAR ipHostNumberBuffer[512];
	mbstowcs_s(0, ipHostNumberBuffer, 512, ipHostNumberString.c_str(), 512);
	PWCHAR ipHostNumber_values[] = {ipHostNumberBuffer, NULL};
	ipHostNumber.mod_op = LDAP_MOD_ADD;
	ipHostNumber.mod_type = L"ipHostNumber";
	ipHostNumber.mod_values = ipHostNumber_values;

	PWCHAR paePwdEncryption_values[] = {L"KERB5", NULL};
	paePwdEncryption.mod_op = LDAP_MOD_ADD;
	paePwdEncryption.mod_type = L"pae-pwdEncryption";
	paePwdEncryption.mod_values = paePwdEncryption_values;

	PWCHAR paeClientProtVersion_values[] = {L"1", NULL};
	paeClientProtVersion.mod_op = LDAP_MOD_ADD;
	paeClientProtVersion.mod_type = L"pae-ClientProtVersion";
	paeClientProtVersion.mod_values = paeClientProtVersion_values;

	PWCHAR paeWinDomain_values[] = {L"NULL", NULL};
	paeWinDomain.mod_op = LDAP_MOD_ADD;
	paeWinDomain.mod_type = L"pae-WinDomain";
	paeWinDomain.mod_values = paeWinDomain_values;

	PWCHAR paeThinProto_values[] = {L"XP_RDP", NULL};
	paeThinProto.mod_op = LDAP_MOD_ADD;
	paeThinProto.mod_type = L"pae-thinProto";
	paeThinProto.mod_values = paeThinProto_values;

	PWCHAR paeServices_values[] = {L"SESSION |, HEARTBEAT |, EVENTS |, USED |", NULL};
	paeServices.mod_op = LDAP_MOD_ADD;
	paeServices.mod_type = L"pae-Services";
	paeServices.mod_values = paeServices_values;

	string datacenterPathString = "/" + desktopSource->getDatacenter() + "/vm/" +desktopSource->getName();
	// convert IP host number to wide char
	WCHAR datacenterPath[512];
	mbstowcs_s(0, datacenterPath, 512, datacenterPathString.c_str(), 512);
	PWCHAR paeVmPath_values[] = {datacenterPath, NULL};
	paeVmPath.mod_op = LDAP_MOD_ADD;
	paeVmPath.mod_type = L"pae-VmPath";
	paeVmPath.mod_values = paeVmPath_values;

	PWCHAR paeOptSuspendTimeout_values[] = {L"0", NULL};
	paeOptSuspendTimeout.mod_op = LDAP_MOD_ADD;
	paeOptSuspendTimeout.mod_type = L"pae-OptSuspendTimeout";
	paeOptSuspendTimeout.mod_values = paeOptSuspendTimeout_values;

	PWCHAR paeOptDisconnectLimitTimeout_values[] = {L"0", NULL};
	paeOptDisconnectLimitTimeout.mod_op = LDAP_MOD_ADD;
	paeOptDisconnectLimitTimeout.mod_type = L"pae-OptDisconnectLimitTimeout";
	paeOptDisconnectLimitTimeout.mod_values = paeOptDisconnectLimitTimeout_values;

	PWCHAR paeOptMaximumSessions_values[] = {L"0", NULL};
	paeOptMaximumSessions.mod_op = LDAP_MOD_ADD;
	paeOptMaximumSessions.mod_type = L"pae-OptMaximumSessions";
	paeOptMaximumSessions.mod_values = paeOptMaximumSessions_values;

	PWCHAR paeDisabled_values[] = {L"0", NULL};
	paeDisabled.mod_op = LDAP_MOD_ADD;
	paeDisabled.mod_type = L"pae-Disabled";
	paeDisabled.mod_values = paeDisabled_values;

	// build the array of attributes
	LDAPMod *newEntry[22];
	newEntry[0] = &name;
	newEntry[1] = &objectClass;
	newEntry[2] = &cn;
	newEntry[3] = &distinguishedName;
	newEntry[4] = &paeOptIgnoreProcessList;
	newEntry[5] = &paeMoid;
	newEntry[6] = &paeServerManaged;
	newEntry[7] = &paeSSOEnabled;
	newEntry[8] = &paeDisplayName;
	newEntry[9] = &paeTunneledConnection;
	newEntry[10] = &ipHostNumber;
	newEntry[11] = &paePwdEncryption;
	newEntry[12] = &paeClientProtVersion;
	newEntry[13] = &paeWinDomain;
	newEntry[14] = &paeThinProto;
	newEntry[15] = &paeServices;
	newEntry[16] = &paeVmPath;
	newEntry[17] = &paeOptSuspendTimeout;
	newEntry[18] = &paeOptDisconnectLimitTimeout;
	newEntry[19] = &paeOptMaximumSessions;
	newEntry[20] = &paeDisabled;
	newEntry[21] = NULL;

	int ret = ldap_add_s(ld, dn, newEntry);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_add_s call failed with 0x%x\n", ret);
			fclose(out);
		}
	}

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Exiting addVM\n");
		fclose(out);
	}
	return ret;
}

int ViewServerCmnd::addServerGroup(int accessMode,
									string serverGroupName, 
									vector<ViewDesktopSource*> &desktopSources,
									LDAP *&ld,
									char* vcdn) {
	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering addServerGroup\n");
		fclose(out);
	}

	string dnString = "CN=" +serverGroupName +",OU=Server Groups,DC=vdi,DC=vmware,DC=int";

	LDAPMod name, objectClass, cn, distinguishedName, paeUserResetAllowed, paeMultiSessionAllowed, paeOptDisconnectLimitTimeout,
		paeServerPoolType, paeVmMinimumCount, paeVmMaximumCount, paeVCDN, paeDisabled, paeMemberDN, paeVmPowerPolicy, paeVmProvEnabled,
		paeVmProvSuspendOnError, paeVmStartClone, paeVmPoolCalculatedValues;

	// convert dn to wide char
	WCHAR dn[512];
	mbstowcs_s(0, dn, 512, dnString.c_str(), 512);

	PWCHAR objectClass_values[] = {L"top", L"pae-ServerPool", NULL};
	objectClass.mod_op = LDAP_MOD_ADD;
	objectClass.mod_type = L"objectClass";
	objectClass.mod_values = objectClass_values;

	// convert server group name to wide char
	WCHAR serverGroupNameBuffer[512];
	mbstowcs_s(0, serverGroupNameBuffer, 512, serverGroupName.c_str(), 512);

	PWCHAR cn_values[] = {serverGroupNameBuffer, NULL};
	cn.mod_op = LDAP_MOD_ADD;
	cn.mod_type = L"cn";
	cn.mod_values = cn_values;

	PWCHAR distinguishedName_values[] = {dn, NULL};
	distinguishedName.mod_op = LDAP_MOD_ADD;
	distinguishedName.mod_type = L"distinguishedName";
	distinguishedName.mod_values = distinguishedName_values;

	PWCHAR name_values[] = {serverGroupNameBuffer, NULL};
	name.mod_op = LDAP_MOD_ADD;
	name.mod_type = L"name";
	name.mod_values = name_values;

	PWCHAR paeDisabled_values[] = {L"0", NULL};
	paeDisabled.mod_op = LDAP_MOD_ADD;
	paeDisabled.mod_type = L"pae-Disabled";
	paeDisabled.mod_values = paeDisabled_values;

	PWCHAR paeOptDisconnectLimitTimeout_values[] = {L"0", NULL};
	paeOptDisconnectLimitTimeout.mod_op = LDAP_MOD_ADD;
	paeOptDisconnectLimitTimeout.mod_type = L"pae-OptDisconnectLimitTimeout";
	paeOptDisconnectLimitTimeout.mod_values = paeOptDisconnectLimitTimeout_values;

	string accessModeString;
	if(accessMode == 0) { // dedicated manual pool
		accessModeString = "5";
	} else { // floating manual pool
		accessModeString = "0";
	}
	// convert access mode to wide char
	WCHAR accessModeBuffer[512];
	mbstowcs_s(0, accessModeBuffer, 512, accessModeString.c_str(), 512);
	PWCHAR paeServerPoolType_values[] = {accessModeBuffer, NULL};
	paeServerPoolType.mod_op = LDAP_MOD_ADD;
	paeServerPoolType.mod_type = L"pae-ServerPoolType";
	paeServerPoolType.mod_values = paeServerPoolType_values;

	PWCHAR paeVmMinimumCount_values[] = {L"0", NULL};
	paeVmMinimumCount.mod_op = LDAP_MOD_ADD;
	paeVmMinimumCount.mod_type = L"pae-VmMinimumCount";
	paeVmMinimumCount.mod_values = paeVmMinimumCount_values;

	PWCHAR paeVmMaximumCount_values[] = {L"0", NULL};
	paeVmMaximumCount.mod_op = LDAP_MOD_ADD;
	paeVmMaximumCount.mod_type = L"pae-VmMaximumCount";
	paeVmMaximumCount.mod_values = paeVmMaximumCount_values;

	string vcdnOnly = string(vcdn);
	string vcdnString = "CN=" +vcdnOnly +",OU=VirtualCenter,OU=Properties,DC=vdi,DC=vmware,DC=int";
	// convert access mode to wide char
	WCHAR vcdnBuffer[512];
	mbstowcs_s(0, vcdnBuffer, 512, vcdnString.c_str(), 512);
	PWCHAR paeVCDN_values[] = {vcdnBuffer, NULL};
	paeVCDN.mod_op = LDAP_MOD_ADD;
	paeVCDN.mod_type = L"pae-VCDN";
	paeVCDN.mod_values = paeVCDN_values;

	// add each member to the pool
	PWCHAR *paeMemberDN_values = new PWCHAR[desktopSources.size() + 1];
	vector<ViewDesktopSource*>::iterator itr;
	int index = 0;
	for(itr = desktopSources.begin(); itr != desktopSources.end(); itr++) {
		ViewDesktopSource *desktopSource = (ViewDesktopSource*)(*itr);
		string sourceDNString = "CN=" +desktopSource->getName() +",OU=Servers,dc=vdi,dc=vmware,dc=int";
		// convert source DN to wide char
		PWCHAR sourceDN = new WCHAR[512];
		mbstowcs_s(0, sourceDN, 512, sourceDNString.c_str(), 512);
		paeMemberDN_values[index++] = sourceDN;
	}
	paeMemberDN_values[desktopSources.size()] = NULL;
	paeMemberDN.mod_op = LDAP_MOD_ADD;
	paeMemberDN.mod_type = L"pae-MemberDN";
	paeMemberDN.mod_values = paeMemberDN_values;

	PWCHAR paeVmPowerPolicy_values[] = {L"remainon", NULL};
	paeVmPowerPolicy.mod_op = LDAP_MOD_ADD;
	paeVmPowerPolicy.mod_type = L"pae-VmPowerPolicy";
	paeVmPowerPolicy.mod_values = paeVmPowerPolicy_values;

	PWCHAR paeVmProvEnabled_values[] = {L"1", NULL};
	paeVmProvEnabled.mod_op = LDAP_MOD_ADD;
	paeVmProvEnabled.mod_type = L"pae-VmProvEnabled";
	paeVmProvEnabled.mod_values = paeVmProvEnabled_values;

	PWCHAR paeVmProvSuspendOnError_values[] = {L"0", NULL};
	paeVmProvSuspendOnError.mod_op = LDAP_MOD_ADD;
	paeVmProvSuspendOnError.mod_type = L"pae-VmProvSuspendOnError";
	paeVmProvSuspendOnError.mod_values = paeVmProvSuspendOnError_values;

	PWCHAR paeVmStartClone_values[] = {L"1", NULL};
	paeVmStartClone.mod_op = LDAP_MOD_ADD;
	paeVmStartClone.mod_type = L"pae-VmStartClone";
	paeVmStartClone.mod_values = paeVmStartClone_values;

	PWCHAR paeVmPoolCalculatedValues_values[] = {L"1", NULL};
	paeVmPoolCalculatedValues.mod_op = LDAP_MOD_ADD;
	paeVmPoolCalculatedValues.mod_type = L"pae-VmPoolCalculatedValues";
	paeVmPoolCalculatedValues.mod_values = paeVmPoolCalculatedValues_values;

	PWCHAR paeUserResetAllowed_values[] = {L"0", NULL};
	paeUserResetAllowed.mod_op = LDAP_MOD_ADD;
	paeUserResetAllowed.mod_type = L"pae-UserResetAllowed";
	paeUserResetAllowed.mod_values = paeUserResetAllowed_values;

	PWCHAR paeMultiSessionAllowed_values[] = {L"0", NULL};
	paeMultiSessionAllowed.mod_op = LDAP_MOD_ADD;
	paeMultiSessionAllowed.mod_type = L"pae-MultiSessionAllowed";
	paeMultiSessionAllowed.mod_values = paeMultiSessionAllowed_values;

	// build the array of attributes
	LDAPMod *newEntry[19];
	newEntry[0] = &objectClass;
	newEntry[1] = &cn;
	newEntry[2] = &distinguishedName;
	newEntry[3] = &name;
	newEntry[4] = &paeDisabled;
	newEntry[5] = &paeOptDisconnectLimitTimeout;
	newEntry[6] = &paeServerPoolType;
	newEntry[7] = &paeVmMinimumCount;
	newEntry[8] = &paeVmMaximumCount;
	newEntry[9] = &paeVCDN;
	newEntry[10] = &paeMemberDN;
	newEntry[11] = &paeVmPowerPolicy;
	newEntry[12] = &paeVmProvEnabled;
	newEntry[13] = &paeVmProvSuspendOnError;
	newEntry[14] = &paeVmStartClone;
	newEntry[15] = &paeVmPoolCalculatedValues;
	newEntry[16] = &paeUserResetAllowed;
	newEntry[17] = &paeMultiSessionAllowed;
	newEntry[18] = NULL;

	int ret = ldap_add_s(ld, dn, newEntry);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_add_s call failed with 0x%x\n", ret);
			fclose(out);
		}
	}

	size_t count = desktopSources.size();
	for(size_t i = 0; i < count; i++) {
		delete[] paeMemberDN_values[i];
	}
	delete[] paeMemberDN_values;

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Exiting addServerGroup\n");
		fclose(out);
	}

	return ret;
}

int ViewServerCmnd::addToExistingServerGroup(string poolName, vector<ViewDesktopSource*> &desktopSources, LDAP *&ld) {

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering addToExistingServerGroup\n");
		fclose(out);
	}

	LDAPMod paeMemberDN;

	string dnString = "CN=" +poolName +",OU=Server Groups,DC=vdi,DC=vmware,DC=int";
	// convert dn to wide char
	WCHAR dn[512];
	mbstowcs_s(0, dn, 512, dnString.c_str(), 512);

	// add each member to the pool
	PWCHAR* paeMemberDN_values = new PWCHAR[desktopSources.size() + 1];
	vector<ViewDesktopSource*>::iterator itr;
	int index = 0;
	for(itr = desktopSources.begin(); itr != desktopSources.end(); itr++) {
		ViewDesktopSource *desktopSource = (ViewDesktopSource*)(*itr);
		string sourceDNString = "CN=" +desktopSource->getName() +",OU=Servers,dc=vdi,dc=vmware,dc=int";
		// convert source DN to wide char
		PWCHAR sourceDN = new WCHAR[512];
		mbstowcs_s(0, sourceDN, 512, sourceDNString.c_str(), 512);
		paeMemberDN_values[index++] = sourceDN;
	}
	paeMemberDN_values[desktopSources.size()] = NULL;
	paeMemberDN.mod_op = LDAP_MOD_ADD;
	paeMemberDN.mod_type = L"pae-MemberDN";
	paeMemberDN.mod_values = paeMemberDN_values;

	// build the array of attributes
	LDAPMod *newEntry[2];
	newEntry[0] = &paeMemberDN;
	newEntry[1] = NULL;

	int ret = ldap_modify_s(ld, dn, newEntry);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_modify_s call failed with 0x%x\n", ret);
			fclose(out);
		}
	}

	// clean up
	size_t count = desktopSources.size();
	for(size_t i = 0; i < count; i++) {
		delete[] paeMemberDN_values[i];
	}
	delete[] paeMemberDN_values;

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Exiting addToExistingServerGroup\n");
		fclose(out);
	}

	return ret;
}

int ViewServerCmnd::addApplication(string applicationName, LDAP *&ld) {
	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Entering addApplication\n");
		fclose(out);
	}

	string dnString = "CN=" +applicationName +",OU=Applications,DC=vdi,DC=vmware,DC=int";

	// convert dn to wide char
	WCHAR dn[512];
	mbstowcs_s(0, dn, 512, dnString.c_str(), 512);

	LDAPMod objectClass, cn, distinguishedName, name, paeServerProtocolLevel, paeMarkedForDelete, paeDisabled,
		paeIcon, paeURL, paeServers;

	PWCHAR objectClass_values[] = {L"top", L"pae-Entity", L"pae-App", L"pae-WinApp", L"pae-ThinWinApp", 
		L"pae-DesktopApplication", NULL};
	objectClass.mod_op = LDAP_MOD_ADD;
	objectClass.mod_type = L"objectClass";
	objectClass.mod_values = objectClass_values;

	// convert application name to wide char
	WCHAR applicationNameBuffer[512];
	mbstowcs_s(0, applicationNameBuffer, 512, applicationName.c_str(), 512);
	PWCHAR cn_values[] = {applicationNameBuffer, NULL};
	cn.mod_op = LDAP_MOD_ADD;
	cn.mod_type = L"cn";
	cn.mod_values = cn_values;

	PWCHAR distinguishedName_values[] = {dn, NULL};
	distinguishedName.mod_op = LDAP_MOD_ADD;
	distinguishedName.mod_type = L"distinguishedName";
	distinguishedName.mod_values = distinguishedName_values;

	PWCHAR name_values[] = {applicationNameBuffer, NULL};
	name.mod_op = LDAP_MOD_ADD;
	name.mod_type = L"name";
	name.mod_values = name_values;

	PWCHAR* paeServerProtocolLevel_values = new PWCHAR[3];
	paeServerProtocolLevel_values[0] = L"PCOIP";
	paeServerProtocolLevel_values[1] = L"RDP";
	paeServerProtocolLevel_values[2] = NULL;
	paeServerProtocolLevel.mod_op = LDAP_MOD_ADD;
	paeServerProtocolLevel.mod_type = L"pae-ServerProtocolLevel";
	paeServerProtocolLevel.mod_values = paeServerProtocolLevel_values;

	PWCHAR paeMarkedForDelete_values[] = {L"0", NULL};
	paeMarkedForDelete.mod_op = LDAP_MOD_ADD;
	paeMarkedForDelete.mod_type = L"pae-MarkedForDelete";
	paeMarkedForDelete.mod_values = paeMarkedForDelete_values;

	PWCHAR paeDisabled_values[] = {L"0", NULL};
	paeDisabled.mod_op = LDAP_MOD_ADD;
	paeDisabled.mod_type = L"pae-Disabled";
	paeDisabled.mod_values = paeDisabled_values;

	PWCHAR paeIcon_values[] = {L"/thinapp/icons/desktop.gif", NULL};
	paeIcon.mod_op = LDAP_MOD_ADD;
	paeIcon.mod_type = L"pae-Icon";
	paeIcon.mod_values = paeIcon_values;

	PWCHAR paeURL_values[] = {L"\\", NULL};
	paeURL.mod_op = LDAP_MOD_ADD;
	paeURL.mod_type = L"pae-URL";
	paeURL.mod_values = paeURL_values;

	string serverNameString = "CN=" +applicationName +",OU=Server Groups,DC=vdi,DC=vmware,DC=int";
	// convert dn to wide char
	WCHAR serverName[512];
	mbstowcs_s(0, serverName, 512, serverNameString.c_str(), 512);
	PWCHAR paeServers_values[] = {serverName, NULL};
	paeServers.mod_op = LDAP_MOD_ADD;
	paeServers.mod_type = L"pae-Servers";
	paeServers.mod_values = paeServers_values;

	// build the array of attributes
	LDAPMod *newEntry[11];
	newEntry[0] = &objectClass;
	newEntry[1] = &cn;
	newEntry[2] = &distinguishedName;
	newEntry[3] = &name;
	newEntry[4] = &paeServerProtocolLevel;
	newEntry[5] = &paeMarkedForDelete;
	newEntry[6] = &paeDisabled;
	newEntry[7] = &paeIcon;
	newEntry[8] = &paeURL;
	newEntry[9] = &paeServers;
	newEntry[10] = NULL;

	int ret = ldap_add_s(ld, dn, newEntry);
	if(ret != LDAP_SUCCESS) {
		if(DEBUG) {
			FILE* out;
			fopen_s(&out, "debug.log", "a");
			fprintf(out, "ldap_add_s call failed with 0x%x\n", ret);
			fclose(out);
		}
	}

	//clean up
	delete[] paeServerProtocolLevel_values;

	if(DEBUG) {
		FILE* out;
		fopen_s(&out, "debug.log", "a");
		fprintf(out, "Exiting addApplication\n");
		fclose(out);
	}

	return ret;
}
